/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


//  Title: curstst.cpp
//
//  Description:
//    Cursor example - open a cursor and fetch next, prev, absolute, relative
//

#include <iostream.h>
#include <it.h>

ITCallbackResult
my_error_handler(const ITErrorManager &errorobject,
                 void *userdata,
                 long errorlevel)
{
    // Cast the user data into a stream
    ostream *stream = (ostream *) userdata;
    (*stream) << "my_error_handler: errorlevel="
	      << errorlevel
	      << " sqlstate="
              << errorobject.SqlState()
              << ' '
              << errorobject.ErrorText()
              << endl;
    return IT_NOTHANDLED;
}

int
main(int, char **)
{
    // Make a connection using defaults					
    ITConnection conn;
    conn.Open();

    // Create a query object
    ITCursor cursor(conn);      
    char qtext[1024];
    
    cout << "> ";

    // Add a callback to the cursor object
    cursor.AddCallback(my_error_handler, (void *) &cerr);

    while (cin.getline(qtext, sizeof(qtext)))
    {
        if (!cursor.Prepare(qtext))
        {
            cout << "Could not prepare cursor for " << qtext << endl;
	        if (cursor.Error())
		        cout << "Error: " << cursor.ErrorText() << endl;
            continue;
        }                         

        if (!cursor.Open(ITCursor::Scrollable+ITCursor::ReadOnly))
        {
            cout << "Could not open query: " << qtext << endl;
        }                                                        
        else
            {
            ITValue *rowValue;
            ITRow *row;
            int rowcount = 0;

            cout << "FORWARDS" << endl;                             // 1 begin
            while ((rowValue = cursor.Fetch()) != NULL)         
            {
                rowcount++;
                cout << rowValue->Printable() << endl;
                rowValue->Release();
            }                                                       // 1 end

            // should be at end now
            cout << "BACKWARDS" << endl;                            // 2 begin
            for (;;)
            {
                if ( !(row = cursor.NextRow( 0, ITPositionPrior )) )
                    break;
                rowcount++;
                cout << row->Printable() << endl;
                row->Release();
            }                                                       // 2 end

            cout << "EVERY SECOND" << endl;                         // 3 begin
            for (;;)
            {
                if ( !(row = cursor.NextRow( 0, ITPositionRelative, 2 )) )
                    break;
                rowcount++;
                cout << row->Printable() << endl;
                row->Release();
            }                                                       // 3 end

            cout << "THIRD" << endl;                                // 4 begin
            row = cursor.NextRow( 0, ITPositionAbsolute, 3 );
            if (row != NULL)
            {
                rowcount++;
                cout << row->Printable() << endl;
                row->Release();
            }                                                       // 4 end
            
            cout << "FIRST" << endl;                                // 5 begin
            row = cursor.NextRow( 0, ITPositionFirst );
            if (row != NULL)
            {
                rowcount++;
                cout << row->Printable() << endl;
                row->Release();
            }                                                       // 5 end
            
            cout << "LAST" << endl;                                 // 6 begin
            row = cursor.NextRow( 0, ITPositionLast );
            if (row != NULL)
            {
                rowcount++;
                cout << row->Printable() << endl;
                row->Release();
            }                                                       // 6 end
            
            cout << "500th" << endl;                                // 7 begin
            row = cursor.NextRow( 0, ITPositionAbsolute, 500 );
            if (row != NULL)
            {
                rowcount++;
                cout << row->Printable() << endl;
                row->Release();
            }                                                       // 7 end

            cout << rowcount << " rows received" << endl;

            if (!cursor.Close())
            {
                cout << "Could not close cursor" << endl;
            }                    
        }
        
        cout << "> ";
    }
    cout << endl;
    conn.Close();

    return 1;
}
