/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


//  Title: loex2.cpp
//  Description:
//	 Large object example, with error info handling

#include <stdio.h>
#include <iostream.h>
#include <it.h>

#ifdef IT_WINDOWS
#include <fcntl.h>
#include <io.h>
#endif

int
main(int, char **)
{
    // Make a connection using defaults
    ITConnection conn;
    conn.Open();

    // Create a query object
    ITQuery q(conn);

    // Retrieve a row from the database
    ITRow *row;           
    row = q.ExecOneRow("select large_object_val from regresstab;");

    // Check to see if any rows were returned
    if (row == NULL)
    {
        cout << "loex2: no rows returned by ITQuery::ExecOneRow" << endl;
        cout << "loex2: exiting" << endl;
        return 1;
    }

    ITValue *v = row->Column(0);
    ITLargeObject *loif;
    ITErrorInfo *errif;                                             // 1 begin
                                                                    // 1 end
    // Extract the interface. The return code IT_QUERYINTERFACE_SUCCESS
    // should be used for compatibility reasons.
    if (v->QueryInterface(ITLargeObjectIID, (void **) &loif)        // 1 begin
        == IT_QUERYINTERFACE_SUCCESS)
    {                                                               // 1 end

        // Extract the errorinfo interface.                         // 2 begin
        if (v->QueryInterface(ITErrorInfoIID, (void **) &errif)
            == IT_QUERYINTERFACE_SUCCESS)
        {							    // 2 end
            // Induce an error. Close the connection, then try reading
            // the large object.
            conn.Close();                                           // 3 begin
                                                                    // 3 end
            long size = 0;
            long n;
            char buf[1024];
        
#ifdef IT_WINDOWS
            _setmode(_fileno(stdout), _O_BINARY);
#endif

            while ((n = loif->Read(buf, sizeof(buf))) > 0)
            {
                size += n;
                cout.write(buf, n);
            }
            cout.flush();

            if (size == 0)                                          // 4 begin
            {
                // No bytes were read. Was there an error?
                if (errif->Error())                                  
                {
                    cerr << "Zero bytes read. Server error was" << endl
                         << errif->ErrorText() << endl;
                }
            }                                                       // 4 end
            else
            {
#ifdef IT_WINDOWS
                _setmode(_fileno(stdout), _O_TEXT);
#endif
                cout << endl << "EOF" << endl;
            }
            
            errif->Release();
        }							   // 2 begin
        							   // 2 end
        
        // Release the large object interface
        loif->Release();
    }                                                               // 1 begin
                                                                    // 1 end
    // Release the value interfaces
    v->Release();
    row->Release();

    return 0;
}


