/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


//  Title: rowref.cpp
//
//  Description:
//	 Multiple interface value object example

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#include <it.h>
#ifndef IT_WINDOWS
#include <strstream.h>
#else
#include <strstrea.h>
#endif

#define NBITS  32

typedef mi_integer bitarray_t;

// Multiple-interface nested class version of the Bitarray
// object from simpval.cpp 
class Bitarray                                                   // 1 begin
{                                                                // 1 end
public:
    // ITDatum-derived nested class. This just passes work through
    // the parent pointer into the parent object
    class XITDatum : public ITDatum
    {
    public:
        // Overrides of ITEssential methods
        virtual ITOpErrorCode IT_STDCALL QueryInterface
			(const ITInterfaceID &ifiid, void **resultif);
        virtual unsigned long IT_STDCALL AddRef();
        virtual unsigned long IT_STDCALL Release();

        // Overrides of ITValue methods
        virtual const ITString & IT_STDCALL Printable();
        virtual const ITTypeInfo & IT_STDCALL TypeOf();
        virtual ITBool IT_STDCALL IsNull();
        virtual ITBool IT_STDCALL SameType(ITValue *);
        virtual ITBool IT_STDCALL CompatibleType(ITValue *);
        virtual ITBool IT_STDCALL Equal(ITValue *);
        virtual ITBool IT_STDCALL LessThan(ITValue *);
        virtual ITBool IT_STDCALL IsUpdated();
        virtual ITBool IT_STDCALL FromPrintable( const ITString & );
        virtual ITBool IT_STDCALL SetNull();
 
        // ITDatum methods
        virtual MI_DATUM     IT_STDCALL Data();
        virtual long         IT_STDCALL DataLength();
        virtual ITBool       IT_STDCALL SetData
            (MI_DATUM, long, ITPreserveData *);
        virtual const ITConnection& IT_STDCALL Connection();
 
        Bitarray *parent;
    } datum_interface;

    // ITContainCvt-derived nested class. This just passes work through
    // the parent pointer into the parent object
    class XITContainCvt : public ITContainCvt
    {
    public:
        // Overrides of ITEssential methods
        virtual ITOpErrorCode IT_STDCALL QueryInterface(const ITInterfaceID &ifiid,
                                                        void **resultif);
        virtual unsigned long IT_STDCALL AddRef();
        virtual unsigned long IT_STDCALL Release();

        // Overrides of ITContainCvt methods
        virtual long IT_STDCALL NumItems();

        // Note that only double conversions are supported. This kind
        // of violates putting implementation in the passthrough
        // class, but it is expedient.
        virtual ITBool IT_STDCALL ConvertTo(long, short &)           { return FALSE; }
        virtual ITBool IT_STDCALL ConvertFrom(long, short)           { return FALSE; }
        virtual ITBool IT_STDCALL ConvertTo(long, int &);
        virtual ITBool IT_STDCALL ConvertFrom(long, int);
        virtual ITBool IT_STDCALL ConvertTo(long, long &)            { return FALSE; }
        virtual ITBool IT_STDCALL ConvertFrom(long, long)            { return FALSE; }
        virtual ITBool IT_STDCALL ConvertTo(long, float &)           { return FALSE; }
        virtual ITBool IT_STDCALL ConvertFrom(long, float)           { return FALSE; }
        virtual ITBool IT_STDCALL ConvertTo(long, double &)          { return FALSE; }
        virtual ITBool IT_STDCALL ConvertFrom(long, double)          { return FALSE; }
        virtual ITBool IT_STDCALL ConvertTo(long, long double &)     { return FALSE; }
        virtual ITBool IT_STDCALL ConvertFrom(long, long double)     { return FALSE; }
        virtual ITBool IT_STDCALL ConvertTo(long, const char * &)    { return FALSE; }
        virtual ITBool IT_STDCALL ConvertFrom(long, const char *)    { return FALSE; }
        virtual ITBool IT_STDCALL ConvertTo(long, ITString &)        { return FALSE; }
        virtual ITBool IT_STDCALL ConvertFrom(long, const ITString &){ return FALSE; }
        virtual ITBool IT_STDCALL ConvertTo(long, ITInt8 &)          { return FALSE; }
        virtual ITBool IT_STDCALL ConvertFrom(long, ITInt8)          { return FALSE; }
#ifdef IT_COMPILER_HAS_BOOL
        virtual ITBool IT_STDCALL ConvertTo(long, bool &)           { return FALSE; }
        virtual ITBool IT_STDCALL ConvertFrom(long, bool)           { return FALSE; }
#endif

        Bitarray *parent;
    } containcvt_interface;
    
    // Class constructor, destructor
    Bitarray(ITMVDesc *);
    ~Bitarray();
    
    // Factory Constructor
    static ITValue *MakeValue(ITMVDesc *);

    // Data members to implement ITEssential functionality
    long refcount;

    // Data members to implement ITValue functionality
    ITTypeInfo typeinfo;
    ITBool isnull, isupdated;
    ITString printable_value;

    // Data members to implement bitarray storage
    // Note that object cannot modify original datum so if object
    // stores pointer and is updated it has to allocate memory
    // for a new value.
    // Note also that since SetNull() sets isupdated if the object
    // was not updated before calling SetNull(), pointer to the datum 
    // has to be set to 0, otherwise destructor will attempt to free
    // the memory that does not belong to the object.
    bitarray_t *pvalue;

    // Extracted from the ITMVDesc object
    ITPreserveData *preservedata;	// reference counter on datum   // 1 begin
                                                                    // 1 end
    long columnno;              // column of object from parent composite
    ITConnection conn;          // connection
    ITConnectionStamp stamp;    // connection stamp                 // 1 begin
                                                                    // 1 end
    // Real implementations of overrides
    virtual ITOpErrorCode IT_STDCALL QueryInterface(const ITInterfaceID &iid, void **ifptr);
    virtual unsigned long IT_STDCALL AddRef();
    virtual unsigned long IT_STDCALL Release();

    virtual const ITString & IT_STDCALL Printable();
    virtual const ITTypeInfo & IT_STDCALL TypeOf();
    virtual ITBool IT_STDCALL IsNull();
    virtual ITBool IT_STDCALL SameType(ITValue *);
    virtual ITBool IT_STDCALL CompatibleType(ITValue *);
    virtual ITBool IT_STDCALL Equal(ITValue *);
    virtual ITBool IT_STDCALL LessThan(ITValue *);
    virtual ITBool IT_STDCALL IsUpdated();
    virtual ITBool IT_STDCALL FromPrintable( const ITString & );
    virtual ITBool IT_STDCALL SetNull();

    virtual long IT_STDCALL NumItems();
    virtual ITBool IT_STDCALL ConvertTo(long item, int &value);
    virtual ITBool IT_STDCALL ConvertFrom(long item, int value);

    // Datum implementations
    virtual MI_DATUM            IT_STDCALL Data();
    virtual long                IT_STDCALL DataLength();
    virtual ITBool              IT_STDCALL SetData
        (MI_DATUM data, long dataLen, ITPreserveData *preservedata);
    virtual const ITConnection& IT_STDCALL Connection();

};                                                                  // 1 begin
                                                                    // 1 end
// Implementation
Bitarray::Bitarray(ITMVDesc *mv)                                    // 2 begin
    : refcount(1),
      typeinfo(*mv->vf_origtypeinfo),
      conn(*mv->vf_connection),
      stamp(mv->vf_connection->GetStamp()),
      preservedata(mv->vf_preservedata),
      isupdated( FALSE ),
      pvalue( 0 )
{
    // NULL?
    isnull = mv->vf_libmivaluetype == MI_NULL_VALUE;

    // set up interfaces
    datum_interface.parent = this;
    containcvt_interface.parent = this;

    if( !isnull )
        {
        pvalue = (bitarray_t *)mv->vf_data;
        // We are holding an outstanding reference to datum, so 
        // increment its owner's reference count. 
        // Note that preservedata can be null for null objects.
        preservedata->AddRef();                                     // 2 end
        }
}                                                                   // 2 begin
                                                                    // 2 end
Bitarray::~Bitarray()                                               // 3 begin
{
    if( isupdated )
        delete pvalue;
    else if( preservedata )
        preservedata->Release();
}                                                                   // 3 end

// Factory
ITFactoryList BitarrayFactory("bitarray", Bitarray::MakeValue);

// Create the Bitarray object, and return pointer to 
// it's ITValue implementation
ITValue *
Bitarray::MakeValue(ITMVDesc *mv)
{
    Bitarray *impl = new Bitarray(mv);
    return (ITValue *)&impl->datum_interface;
}

// Overrides

ITOpErrorCode
Bitarray::QueryInterface(const ITInterfaceID &iid,
                                void **ifptr)
{
    int result = IT_QUERYINTERFACE_SUCCESS;
    
    // Return different interfaces as appropriate by referencing
    // nested class members.
    switch (ITIIDtoSID(iid))
    {
        case ITEssentialSID:
        case ITValueSID:
        case ITDatumSID:
            *ifptr = (void *) &datum_interface;
            break;

        case ITContainCvtSID:
            *ifptr = (void *) &containcvt_interface;
            break;
            
        default:
            result = IT_QUERYINTERFACE_FAILED;
            *ifptr = NULL;
            break;
    }
    if (IT_QUERYINTERFACE_SUCCESS == result)
        AddRef();

    return result;
}

unsigned long
Bitarray::AddRef()
{
    return ++refcount;
}

unsigned long
Bitarray::Release()
{
    if (--refcount <= 0)
    {
        delete this;
        return 0;
    }
    else
    {
        return refcount;
    }
}

// Overrides of ITValue methods
const ITString &                                                        // 4 begin
Bitarray::Printable()
{
    // If the underlying data has changed its not safe to proceed.
    if (!stamp.EqualConnInstance(conn.GetStamp()))
        return ITString::Null;
    
    if( IsNull() )
      return printable_value = "null";

    char buf[32];
    ostrstream cstream( buf, sizeof buf );
    cstream << *pvalue << ends;
    return printable_value = cstream.str();
}                                                                       // 4 end

ITBool 
Bitarray::FromPrintable( const ITString &val )
{
    if( !isupdated )
        {
        bitarray_t  *p = new bitarray_t;
        if( !p )
            return FALSE;
        pvalue = p;
        if( preservedata )
            {
            preservedata->Release();
            preservedata = 0;
            }
        }
    if( val.IsNull() )
        isnull = TRUE;
    else
        {
        istrstream cstream( (char *)val.Data(), val.Size() );
        cstream >> *pvalue;
        isnull = FALSE;
        }
    return isupdated = TRUE;
}

const ITTypeInfo &
Bitarray::TypeOf()
{
    return typeinfo;
}

ITBool
Bitarray::IsNull()
{
    return isnull;
}

ITBool
Bitarray::SameType(ITValue *v)
{
    return TypeOf().Name().Equal(v->TypeOf().Name());
}

ITBool
Bitarray::CompatibleType(ITValue *)
{
    return TRUE;
}

ITBool
Bitarray::Equal(ITValue *v)
{
    return Printable().Equal(v->Printable());
}

ITBool
Bitarray::LessThan(ITValue *)
{
    return false;
}

ITBool 
Bitarray::IsUpdated()
{
    return isupdated;
}
 
ITBool 
Bitarray::SetNull()
{
    if( !isupdated )
        {
        pvalue = 0;
        if( preservedata )
            {
            preservedata->Release();
            preservedata = 0;
            }
        }
    return isupdated = isnull = TRUE;
}

// ContainCvt implementation
ITBool
Bitarray::ConvertTo(long item, int &dbvalue)
{
    if ( IsNull() || item >= NumItems())
        return false;
    dbvalue = !!(*pvalue & (1 << (NBITS - 1 - item)));
    return true;
}

long
Bitarray::NumItems()
{
  return IsNull() ? 0 : NBITS;
}

ITBool
Bitarray::ConvertFrom( long item, int val )
{
    if( NumItems() <= item )
        return FALSE;
    if( !isupdated )
        {
        bitarray_t  *p = new bitarray_t;
        if( !p )
            return FALSE;
        pvalue = p;
        if( preservedata )
            {
            preservedata->Release();
            preservedata = 0;
            }
        }
    *pvalue = val ? *pvalue | (1 << (NBITS - 1 - item))
        : *pvalue & ~(1 << (NBITS - 1 - item));
    isnull = FALSE;
    return isupdated = TRUE;
}

//
// ITDatum implementations
//
 
MI_DATUM
Bitarray::Data()
{
    return pvalue;
}
 
long
Bitarray::DataLength()
{
    return sizeof(bitarray_t);
}
 
ITBool
Bitarray::SetData(MI_DATUM data, long, ITPreserveData *)
{
    if( !isupdated )
        {
        bitarray_t  *p = new bitarray_t;
        if( !p )
            return FALSE;
        pvalue = p;
        if( preservedata )
            {
            preservedata->Release();
            preservedata = 0;
            }
        }
    *pvalue = *(bitarray_t *)data;
    isnull = FALSE;
    return isupdated = TRUE;
}
 
const ITConnection&
Bitarray::Connection()
{
    return conn;
}

// NESTED CLASS PASSTHROUGHS
ITOpErrorCode
Bitarray::XITDatum::QueryInterface(const ITInterfaceID &ifiid,
                                          void **resultif)
{
    return parent->QueryInterface(ifiid, resultif);
}

unsigned long
Bitarray::XITDatum::AddRef()
{
    return parent->AddRef();
}

unsigned long
Bitarray::XITDatum::Release()
{
    return parent->Release();
}

const ITString &
Bitarray::XITDatum::Printable()
{
    return parent->Printable();
}

const ITTypeInfo &
Bitarray::XITDatum::TypeOf()
{
    return parent->TypeOf();
}

ITBool
Bitarray::XITDatum::IsNull()
{
    return parent->IsNull();
}

ITBool
Bitarray::XITDatum::SameType(ITValue *v)
{
    return parent->SameType(v);
}

ITBool
Bitarray::XITDatum::CompatibleType(ITValue *v)
{
    return parent->CompatibleType(v);
}

ITBool
Bitarray::XITDatum::Equal(ITValue *v)
{
    return parent->Equal(v);
}

ITBool
Bitarray::XITDatum::LessThan(ITValue *v)
{
    return parent->LessThan(v);
}

ITBool
Bitarray::XITDatum::IsUpdated()
{
    return parent->IsUpdated();
}

ITBool
Bitarray::XITDatum::FromPrintable( const ITString &v )
{
    return parent->FromPrintable( v );
}

ITBool
Bitarray::XITDatum::SetNull()
{
    return parent->SetNull();
}

ITOpErrorCode
Bitarray::XITContainCvt::QueryInterface(const ITInterfaceID &ifiid,
                                               void **resultif)
{
    return parent->QueryInterface(ifiid, resultif);
}

unsigned long
Bitarray::XITContainCvt::AddRef()
{
    return parent->AddRef();
}

unsigned long
Bitarray::XITContainCvt::Release()
{
    return parent->Release();
}

ITBool
Bitarray::XITContainCvt::ConvertTo(long item, int &value)
{
    return parent->ConvertTo(item, value);
}

long
Bitarray::XITContainCvt::NumItems()
{
    return parent->NumItems();
}

ITBool
Bitarray::XITContainCvt::ConvertFrom(long item, int value)
{
    return parent->ConvertFrom(item, value);
}

// Overrides of ITDatum methods

MI_DATUM
Bitarray::XITDatum::Data()
{
    return parent->Data();
}
 
long
Bitarray::XITDatum::DataLength()
{
    return parent->DataLength();
}
 
ITBool
Bitarray::XITDatum::SetData
	(MI_DATUM data, long datalen, ITPreserveData *preservedata)
{
    return parent->SetData(data, datalen, preservedata);
}
 
const ITConnection&
Bitarray::XITDatum::Connection()
{
    return parent->Connection();
}

