/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


//  Title: rowset.cpp
//
//  Description:
//	 Illustrates query rowset functionality. 
//   Note that connection can be closed after the rowset is created.

#include <iostream.h>
#include <it.h>

int
main(int, char **)
{
    // Use default database
    ITConnection conn;

    cout << "> ";
    
    // Get SQL from stdin, open the connection, run the query and than close
    // the connection again.
    char qtext[1024];
    while (cin.getline(qtext, 1024))
        {
        conn.Open();
        if (conn.Error())
            {
            cerr << conn.ErrorText() << endl;
            return 1;
            }
        ITQuery q(conn);

        ITSet *set = q.ExecToSet(qtext);                             // 1 begin
        if (set == NULL)
            {                                                        // 1 end
            cout << "Could not execute query: " << qtext << endl;
            cout << "> ";
            continue;
            }

        ITTypeInfo ti(set->TypeOf());
        cout << "Type of rows : " << ti.Name() << endl;

        // No longer need to be connected
        if( !conn.Close() )
            {
            cout << "Could not close the connection" << endl;
            break;
            }

        // Open the set
        if (!set->Open())                                            // 2 begin
            {                                                        // 2 end
            cout << "Could not open set for query: " << qtext << endl;
            }                                                        // 2 begin
                                                                     // 2 end
        else
            {
            ITValue *value;
            ITRow *row;
            int rowcount = 0;
            
            while ((value = set->Fetch()) != NULL)                   // 3 begin
                {                                                    // 3 end

                if (value->QueryInterface(ITRowIID, (void **) &row)  // 4 begin
                    == IT_QUERYINTERFACE_FAILED)
                    {
                    cout << "Could not get row interface..." << endl;
                    }
                else
                    {
                    cout << row->Printable() << endl;
                    row->Release();
                    }
                rowcount++;
                value->Release();                                     // 4 end
            }
            cout << rowcount << " rows received" << endl;
            
            if (!set->Close())                                       // 5 begin
                {                                                    // 5 end
                cout << "Could not close set" << endl;
                }                                                    // 5 begin
                                                                     // 5 end
            }
        
        set->Release();                                              // 6 begin
                                                                     // 6 end
        cout << "> ";
        }
    cout << endl;
    
    return 0;
}
